#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *

#import pymysql
import dm

class MyTable(QTableWidget):
    def __init__(self, *args):
        QTableWidget.__init__(self, *args)
        self.resizeColumnsToContents()
        self.horizontalHeader().setSortIndicatorShown(True)
        self.horizontalHeader().setStretchLastSection(True)
        self.horizontalHeader().setCascadingSectionResizes(True)
        # равномерное изменение ширины столбцов
        #self.horizontalHeader().setSectionResizeMode(1)
        # изменение ширины столбцов по размеру текста
        #self.horizontalHeader().setSectionResizeMode(3)
        self.horizontalHeader().setStyleSheet("color: black; font-weight:bold; background-color: lightblue;")
        self.setWordWrap(True)
        self.setSortingEnabled(True)

        # обработка нажатия мышой на таблице
        self.doubleClicked.connect(self.on_click)
       # self.doubleClicked.connect(self.contextMenuEvent(self.cursor()))

    # показ контекстного меню
    def contextMenuEvent(self, event):
        Rmenu = QMenu(self)
        addRecord = Rmenu.addAction("Добавить запись")
        delRecord = Rmenu.addAction("Удалить запись")
        relations = Rmenu.addAction("Показать связанные документы")
        action = Rmenu.exec_(self.mapToGlobal(event.pos()))
        if action == delRecord:
            row = self.rowAt(event.pos().y())
            col = self.columnAt(event.pos().x())
            print("Удаляем row:%d, col:%d" % (row, col))
            item = self.item(self.currentIndex().row(), self.currentIndex().column())
            print("Запись" + item.text())
        elif action == addRecord:
            print("Добавить запись  ")
            addNewRecord()

    def on_click(self):
        print(self.cursor().pos().x())
        #self.contextMenuEvent(self.cursor())
        for currentQTableWidgetItem in self.selectedItems():
            print(currentQTableWidgetItem.row(), currentQTableWidgetItem.column(), currentQTableWidgetItem.text())

# Форма для добавления и редактирования
class EditForm(QMainWindow):
    lblList = []
    editList = []
    def __init__(self, tblName, tblDescr):
        super().__init__()
        self.tblName = tblName
        self.setWindowTitle("Добавление записи")
        scroll_widget = QWidget()
        self.general_layout = QVBoxLayout()
        scroll_area = QScrollArea()

        l = QVBoxLayout()
        headBox = QHBoxLayout()
        lblHeader = QLabel("<B>Таблица: " + tblDescr)
        l.addLayout(headBox)
        #palet = QPalette(lblHeader.palette())
        #lblHeader.setPalette(palet)
        headBox.addWidget(lblHeader)
        hboxBtn = QHBoxLayout()
        btnOk = QPushButton('Записать')
        btnOk.clicked.connect(lambda: self.save())
        btnCancel = QPushButton('Закрыть')
        btnCancel.clicked.connect(lambda: self.close())
        # цифровой валидатор, для проверки ввода
        integerRegex = QRegExp("^[0-9]+$")
        integerValidator = QRegExpValidator(integerRegex, self)
        # список соответствия полей и QEdit
        self.widgetsList = []
        # добавляем динамически поля на форму в зависимости от типов данных
        for item in dm.getFields(tblName):
            #print(item)
            fName = item[0]
            fType = item[1]
            fDescr = dm.getFieldDescription(tblName, fName)
            hboxEdit = QHBoxLayout()
            lbl = QLabel(fDescr)
            # создаем поле ввода соответсвующее типу данных
            if fType == 'datetime':
                edit = QDateTimeEdit()
                edit.setCalendarPopup(True)
                edit.setFrame(False)
                now = QDateTime.currentDateTime()
                edit.setDateTime(now)
            elif fType == 'date':
                edit = QDateEdit()
                edit.setCalendarPopup(True)
                edit.setFrame(False)
                now = QDate.currentDate()
                edit.setDate(now)
            elif fType == 'time':
                edit = QTimeEdit()
                edit.setCalendarPopup(True)
                edit.setFrame(False)
                now = QTime.currentTime()
                edit.setTime(now)
            elif fType == 'integer':
                edit = QLineEdit()
                # проверка на ввод цифр
                edit.setValidator(integerValidator)
            else:
                edit = QLineEdit()
            # создаём список полей и соответсвующих им QEdit
            self.widgetsList.append([fName, edit])

            hboxEdit.addWidget(lbl)
            hboxEdit.addWidget(edit)
            l.addLayout(hboxEdit)
        print(self.widgetsList)
        hboxBtn.addWidget(btnOk)
        hboxBtn.addWidget(btnCancel)
        l.addLayout(hboxBtn)

        self.general_layout.addLayout(l)

        scroll_widget.setLayout(self.general_layout)
        scroll_area.setWidget(scroll_widget)
        self.setCentralWidget(scroll_area)

    def save(self):
        listData = [self.tblName]
        listFieldData = []
        for item in self.widgetsList:
            #print(item[0] + ' ' + item[1].text())
            listFieldData.append([item[0], item[1].text()])
        listData.append(listFieldData)
        #print(listData)
        dm.insertDataIntoBD(listData)
        self.close()



class EditForm_(QMainWindow):
    def __init__(self, tblName, tblDescr):
        super().__init__()
        self.setWindowTitle("Добавление записи")
        scroll_widget = QWidget()
        self.general_layout = QVBoxLayout()
        scroll_area = QScrollArea()

        l = QVBoxLayout()
        headBox = QHBoxLayout()
        lblHeader = QLabel("<B>Таблица: " + tblDescr)
        l.addLayout(headBox)
        #palet = QPalette(lblHeader.palette())
        #lblHeader.setPalette(palet)
        headBox.addWidget(lblHeader)
        hboxBtn = QHBoxLayout()
        btnOk = QPushButton('Записать')
        btnOk.clicked.connect(lambda: self.save())
        btnCancel = QPushButton('Закрыть')
        btnCancel.clicked.connect(lambda: self.close())

        # ПРОВЕРКА ВВОДА, добавить в цикл генерации формы в зависимости от типа поля
        # grid_layout.addWidget(QLabel("Введите IP-адрес", self), 0, 0)
        #
        # ipRange = "(?:[0-1]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])"  # Часть регулярного выржение
        # # Само регулярное выражение
        # ipRegex = QRegExp("^" + ipRange + "\\." + ipRange + "\\." + ipRange + "\\." + ipRange + "$")
        # ipValidator = QRegExpValidator(ipRegex, self)  # Валидатор для QLineEdit
        #
        # lineEdit = QLineEdit()
        # lineEdit.setValidator(ipValidator)  # Устанавливаем валидатор

        i=0
        for item in dm.getTableStructure(tblName):
            #print(item)
            hboxEdit = QHBoxLayout()
            lbl = QLabel(item)
            edit = QLineEdit()
            hboxEdit.addWidget(lbl)
            hboxEdit.addWidget(edit)
            l.addLayout(hboxEdit)
            i = i+1

        hboxBtn.addWidget(btnOk)
        hboxBtn.addWidget(btnCancel)
        l.addLayout(hboxBtn)

        self.general_layout.addLayout(l)

        scroll_widget.setLayout(self.general_layout)
        scroll_area.setWidget(scroll_widget)
        self.setCentralWidget(scroll_area)

    def save(self):
        print(self.children())

def addDataIntoTable(tbl_descr, tblDataWidget):
    global tblList
    # выбираем название таблицы по описанию
    for i in tblList:
        if i[1] == tbl_descr:
            tbl_name = i[0]
    #print(tbl_descr)
    data = dm.selectData(tbl_name)
    #print(data)
    fieldNames = dm.getTableStructure(tbl_name)
    print(fieldNames)
    # проверка на наличие записей в таблице
    if data:
        # количество строк
        rows = len(data)
        # КОЛИЧЕСТВО КОЛОНОК
        cols = len(data[0])
    else:
        rows = 0
        cols = len(fieldNames)
    n = 0
    tblDataWidget.setRowCount(rows)
    tblDataWidget.setColumnCount(cols)
    for item in fieldNames:
        fName = item[0]
        fDescr = item[1]
        #fieldNames = item + ','
        # установка заголовков столбцов таблицы
        #item = '<b>' + item + '</b>'
        tblDataWidget.setHorizontalHeaderItem(n, QTableWidgetItem(fDescr))
        n = n + 1

    n = 0
    for key in data:
        m = 0
        for item in key:
            # преобразуем все типы в строку
            newitem = QTableWidgetItem(str(item))
            tblDataWidget.setItem(m, n, newitem)
            n += 1
        m += 1

def showMenuToolbar(window):
    newAction = QAction(QIcon('img/new.gif'), 'Добавить', window)
    newAction.setShortcut('Ins')
    newAction.setStatusTip('Добавить')
    newAction.triggered.connect(addNewRecord)

    deleteAction = QAction(QIcon('img/delete.gif'), 'Удалить', window)
    deleteAction.setShortcut('Del')
    deleteAction.setStatusTip('Удалить')
    # deleteAction.triggered.connect(window.qqqq)

    exitAction = QAction(QIcon('img/exit.gif'), 'Выход', window)
    exitAction.setShortcut('Ctrl+Q')
    exitAction.setStatusTip('Выход')
    exitAction.triggered.connect(window.close)

    cutAction = QAction(QIcon('img/cut.gif'), 'Вырезать', window)
    cutAction.setShortcut('Ctrl+X')
    cutAction.setStatusTip('Вырезать')
    # cutAction.triggered.connect(window.close)

    copyAction = QAction(QIcon('img/copy.gif'), 'Копировать', window)
    copyAction.setShortcut('Ctrl+С')
    copyAction.setStatusTip('Копировать')
    # copyAction.triggered.connect(window.close)

    pasteAction = QAction(QIcon('img/paste.gif'), 'Вставить', window)
    pasteAction.setShortcut('Ctrl+V')
    pasteAction.setStatusTip('Вставить')
    # pasteAction.triggered.connect(window.close)

    findAction = QAction(QIcon('img/find.gif'), 'Копировать', window)
    findAction.setShortcut('Ctrl+F')
    findAction.setStatusTip('Искать')
    # findAction.triggered.connect(window.close)

    printAction = QAction(QIcon('img/print.gif'), 'Печатать', window)
    printAction.setShortcut('Ctrl+P')
    printAction.setStatusTip('Печатать')
    # printAction.triggered.connect(window.close)

    window.statusBar()

    menubar = window.menuBar()
    fileMenu = menubar.addMenu('&Файл')
    fileMenu.addAction(newAction)
    fileMenu.addAction(deleteAction)
    fileMenu.addAction(printAction)
    fileMenu.addAction(exitAction)

    editMenu = menubar.addMenu('&Редактирование')
    editMenu.addAction(copyAction)
    editMenu.addAction(cutAction)
    editMenu.addAction(pasteAction)
    editMenu.addAction(printAction)

    helpMenu = menubar.addMenu('&Помощь')

    # toolbar = window.addToolBar('Редактирование')

    toolbar = window.addToolBar('Панель инструментов')
    toolbar.addAction(newAction)
    toolbar.addAction(deleteAction)
    toolbar.addAction(copyAction)
    toolbar.addAction(cutAction)
    toolbar.addAction(pasteAction)
    toolbar.addAction(findAction)
    toolbar.addAction(printAction)
    # toolbar.addAction(exitAction)

def addNewRecord():
    global listTables, tblRelationsData
    tblDescr = listTables.model().data(listTables.currentIndex())
    for i in tblList:
        if i[1] == tblDescr:
            tblName = i[0]

    print(tblName)
    editForm = EditForm(tblName, tblDescr)
    editForm.show()
    #editForm.setParent(listTables)
    #editForm.show()
    #editForm.setParent(tblRelationsData)
    #editForm = Example()

def main():
    global tblList, listTables, tblRelationsData
    app = QApplication(sys.argv)
    mainWin = QMainWindow()
    workArea = QWidget()
    mainWin.setCentralWidget(workArea)
    hbox = QHBoxLayout()
    listTables = QListWidget()
    listTables.setFrameShape(QFrame.StyledPanel)
    listTables.setFixedWidth(200)
    #tblData = QTableWidget()
    tblData = MyTable()
    tblData.setFrameShape(QFrame.StyledPanel)
    tblRelationsData = QTableWidget()
    tblRelationsData.setFrameShape(QFrame.StyledPanel)
    splitter1 = QSplitter(Qt.Horizontal)
    splitter1.addWidget(listTables)
    splitter2 = QSplitter(Qt.Vertical)
    splitter2.addWidget(tblData)
    splitter2.addWidget(tblRelationsData)
    splitter1.addWidget(splitter2)
    hbox.addWidget(splitter1)
    workArea.setLayout(hbox)
    showMenuToolbar(mainWin)
    listTables.itemClicked.connect(lambda: addDataIntoTable(listTables.model().data(listTables.currentIndex()), tblData))
    # выводим список таблиц в левом поле
    tblList = dm.initDBstructure()
    for i in tblList:
        listTables.addItem(i[1])

    mainWin.setGeometry(300, 300, 800, 600)
    mainWin.setWindowTitle('Data manipulator')
    mainWin.show()

    return app.exec_()
#
#
if __name__ == '__main__':
    main()
